@@ -18,12 +18,14 @@
 package controllers
 
 import (
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
+	zkApi "github.com/pravega/zookeeper-operator/api/v1beta1"
+	policyv1 "k8s.io/api/policy/v1"
+	"regexp"
 
 	solrv1beta1 "github.com/apache/solr-operator/api/v1beta1"
 	"github.com/apache/solr-operator/controllers/util"
-	zk_api "github.com/apache/solr-operator/controllers/zk_api"
 	"golang.org/x/net/context"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -121,8 +123,8 @@ func expectSolrPrometheusExporterWithChecks(ctx context.Context, solrPrometheusE
 	return foundSolrPrometheusExporter
 }
 
-func expectSolrPrometheusExporterWithConsistentChecks(ctx context.Context, solrPrometheusExporter *solrv1beta1.SolrCloud, additionalChecks func(Gomega, *solrv1beta1.SolrCloud), additionalOffset ...int) *solrv1beta1.SolrCloud {
-	foundSolrPrometheusExporter := &solrv1beta1.SolrCloud{}
+func expectSolrPrometheusExporterWithConsistentChecks(ctx context.Context, solrPrometheusExporter *solrv1beta1.SolrPrometheusExporter, additionalChecks func(Gomega, *solrv1beta1.SolrPrometheusExporter), additionalOffset ...int) *solrv1beta1.SolrPrometheusExporter {
+	foundSolrPrometheusExporter := &solrv1beta1.SolrPrometheusExporter{}
 	ConsistentlyWithOffset(resolveOffset(additionalOffset), func(g Gomega) {
 		g.Expect(k8sClient.Get(ctx, resourceKey(solrPrometheusExporter, solrPrometheusExporter.Name), foundSolrPrometheusExporter)).To(Succeed(), "Expected SolrPrometheusExporter does not exist")
 		if additionalChecks != nil {
@@ -133,6 +135,34 @@ func expectSolrPrometheusExporterWithConsistentChecks(ctx context.Context, solrP
 	return foundSolrPrometheusExporter
 }
 
+func expectSolrBackup(ctx context.Context, solrBackup *solrv1beta1.SolrBackup, additionalOffset ...int) *solrv1beta1.SolrBackup {
+	return expectSolrBackupWithChecks(ctx, solrBackup, nil, resolveOffset(additionalOffset))
+}
+
+func expectSolrBackupWithChecks(ctx context.Context, solrBackup *solrv1beta1.SolrBackup, additionalChecks func(Gomega, *solrv1beta1.SolrBackup), additionalOffset ...int) *solrv1beta1.SolrBackup {
+	foundSolrBackup := &solrv1beta1.SolrBackup{}
+	EventuallyWithOffset(resolveOffset(additionalOffset), func(g Gomega) {
+		g.Expect(k8sClient.Get(ctx, resourceKey(solrBackup, solrBackup.Name), foundSolrBackup)).To(Succeed(), "Expected SolrBackup does not exist")
+		if additionalChecks != nil {
+			additionalChecks(g, foundSolrBackup)
+		}
+	}).Should(Succeed())
+
+	return foundSolrBackup
+}
+
+func expectSolrBackupWithConsistentChecks(ctx context.Context, solrBackup *solrv1beta1.SolrBackup, additionalChecks func(Gomega, *solrv1beta1.SolrBackup), additionalOffset ...int) *solrv1beta1.SolrBackup {
+	foundSolrBackup := &solrv1beta1.SolrBackup{}
+	ConsistentlyWithOffset(resolveOffset(additionalOffset), func(g Gomega) {
+		g.Expect(k8sClient.Get(ctx, resourceKey(solrBackup, solrBackup.Name), foundSolrBackup)).To(Succeed(), "Expected SolrBackup does not exist")
+		if additionalChecks != nil {
+			additionalChecks(g, foundSolrBackup)
+		}
+	}).Should(Succeed())
+
+	return foundSolrBackup
+}
+
 func expectSecret(ctx context.Context, parentResource client.Object, secretName string, additionalOffset ...int) *corev1.Secret {
 	return expectSecretWithChecks(ctx, parentResource, secretName, nil, resolveOffset(additionalOffset))
 }
@@ -326,6 +356,47 @@ func expectNoIngress(ctx context.Context, parentResource client.Object, ingressN
 	}).Should(MatchError("ingresses.networking.k8s.io \""+ingressName+"\" not found"), "Ingress exists when it should not")
 }
 
+func expectPodDisruptionBudget(ctx context.Context, parentResource client.Object, podDisruptionBudgetName string, selector *metav1.LabelSelector, maxUnavailable intstr.IntOrString, additionalOffset ...int) *policyv1.PodDisruptionBudget {
+	return expectPodDisruptionBudgetWithChecks(ctx, parentResource, podDisruptionBudgetName, selector, maxUnavailable, nil, resolveOffset(additionalOffset))
+}
+
+func expectNoPodDisruptionBudget(ctx context.Context, parentResource client.Object, podDisruptionBudgetName string, additionalOffset ...int) {
+	podDisruptionBudget := &policyv1.PodDisruptionBudget{}
+	EventuallyWithOffset(resolveOffset(additionalOffset), func(g Gomega) {
+		g.Expect(k8sClient.Get(ctx, resourceKey(parentResource, podDisruptionBudgetName), podDisruptionBudget)).To(MatchError("poddisruptionbudgets.policy \""+podDisruptionBudgetName+"\" not found"), "Expected pdb to eventually not exist")
+	}).Should(Succeed())
+}
+
+func expectPodDisruptionBudgetWithChecks(ctx context.Context, parentResource client.Object, podDisruptionBudgetName string, selector *metav1.LabelSelector, maxUnavailable intstr.IntOrString, additionalChecks func(Gomega, *policyv1.PodDisruptionBudget), additionalOffset ...int) *policyv1.PodDisruptionBudget {
+	podDisruptionBudget := &policyv1.PodDisruptionBudget{}
+	EventuallyWithOffset(resolveOffset(additionalOffset), func(g Gomega) {
+		g.Expect(k8sClient.Get(ctx, resourceKey(parentResource, podDisruptionBudgetName), podDisruptionBudget)).To(Succeed(), "Expected PodDisruptionBudget does not exist")
+
+		// Verify the PodDisruptionBudget Spec
+		g.Expect(podDisruptionBudget.Spec.Selector).To(Equal(selector), "PodDisruptionBudget does not have the correct selector.")
+		g.Expect(podDisruptionBudget.Spec.MaxUnavailable).To(Equal(&maxUnavailable), "PodDisruptionBudget does not have the correct maxUnavailable setting.")
+
+		if additionalChecks != nil {
+			additionalChecks(g, podDisruptionBudget)
+		}
+	}).Should(Succeed())
+
+	By("recreating the PodDisruptionBudget after it is deleted")
+	ExpectWithOffset(resolveOffset(additionalOffset), k8sClient.Delete(ctx, podDisruptionBudget)).To(Succeed())
+	EventuallyWithOffset(
+		resolveOffset(additionalOffset),
+		func() (types.UID, error) {
+			newResource := &policyv1.PodDisruptionBudget{}
+			err := k8sClient.Get(ctx, resourceKey(parentResource, podDisruptionBudgetName), newResource)
+			if err != nil {
+				return "", err
+			}
+			return newResource.UID, nil
+		}).Should(And(Not(BeEmpty()), Not(Equal(podDisruptionBudget.UID))), "New PodDisruptionBudget, with new UID, not created.")
+
+	return podDisruptionBudget
+}
+
 func expectConfigMap(ctx context.Context, parentResource client.Object, configMapName string, configMapData map[string]string, additionalOffset ...int) *corev1.ConfigMap {
 	return expectConfigMapWithChecks(ctx, parentResource, configMapName, configMapData, nil, resolveOffset(additionalOffset))
 }
@@ -506,36 +577,32 @@ func filterVarsByName(envVars []corev1.EnvVar, f func(string) bool) []corev1.Env
 }
 
 func testPodEnvVariables(expectedEnvVars map[string]string, foundEnvVars []corev1.EnvVar, additionalOffset ...int) {
-	testGenericPodEnvVariables(expectedEnvVars, foundEnvVars, "SOLR_OPTS", resolveOffset(additionalOffset))
+	testPodEnvVariablesWithGomega(Default, expectedEnvVars, foundEnvVars, resolveOffset(additionalOffset))
 }
 
 func testPodEnvVariablesWithGomega(g Gomega, expectedEnvVars map[string]string, foundEnvVars []corev1.EnvVar, additionalOffset ...int) {
-	testGenericPodEnvVariablesWithGomega(g, expectedEnvVars, foundEnvVars, "SOLR_OPTS", resolveOffset(additionalOffset))
-}
-
-func testMetricsPodEnvVariables(expectedEnvVars map[string]string, foundEnvVars []corev1.EnvVar, additionalOffset ...int) {
-	testGenericPodEnvVariables(expectedEnvVars, foundEnvVars, "JAVA_OPTS", resolveOffset(additionalOffset))
-}
-
-func testMetricsPodEnvVariablesWithGomega(g Gomega, expectedEnvVars map[string]string, foundEnvVars []corev1.EnvVar, additionalOffset ...int) {
-	testGenericPodEnvVariablesWithGomega(g, expectedEnvVars, foundEnvVars, "JAVA_OPTS", resolveOffset(additionalOffset))
-}
-
-func testGenericPodEnvVariables(expectedEnvVars map[string]string, foundEnvVars []corev1.EnvVar, lastVarName string, additionalOffset ...int) {
-	testGenericPodEnvVariablesWithGomega(Default, expectedEnvVars, foundEnvVars, lastVarName, resolveOffset(additionalOffset))
-}
-
-func testGenericPodEnvVariablesWithGomega(g Gomega, expectedEnvVars map[string]string, foundEnvVars []corev1.EnvVar, lastVarName string, additionalOffset ...int) {
+	envVarRegex := regexp.MustCompile(`\$\([a-zA-Z0-9_]+\)`)
 	offset := resolveOffset(additionalOffset)
 	matchCount := 0
-	for _, envVar := range foundEnvVars {
+	var processedEnvVarNames = make([]string, len(foundEnvVars))
+	for i, envVar := range foundEnvVars {
 		if expectedVal, match := expectedEnvVars[envVar.Name]; match {
 			matchCount += 1
 			g.ExpectWithOffset(offset, envVar.Value).To(Equal(expectedVal), "Wrong value for env variable '%s' in podSpec", envVar.Name)
 		}
+
+		// Check that the current envVar only references other env-vars that have already been defined
+		envVarsReferencedByCurrent := envVarRegex.FindAllString(envVar.Value, -1)
+		for _, referencedVar := range envVarsReferencedByCurrent {
+			referencedVarTrimmed := referencedVar[2 : len(referencedVar)-1] // "$(ENV_VAR_NAME)" -> "ENV_VAR_NAME"
+			g.Expect(processedEnvVarNames).To(ContainElement(referencedVarTrimmed),
+				"Env-var %s with value [%s] must be defined after the env-var it depends on: %s",
+				envVar.Name, envVar.Value, referencedVarTrimmed)
+		}
+
+		processedEnvVarNames[i] = envVar.Name
 	}
 	g.ExpectWithOffset(offset, matchCount).To(Equal(len(expectedEnvVars)), "Not all expected env variables found in podSpec")
-	g.ExpectWithOffset(offset, foundEnvVars[len(foundEnvVars)-1].Name).To(Equal(lastVarName), "%s must be the last envVar set, as it uses other envVars.", lastVarName)
 }
 
 func testMapContainsOther(mapName string, base map[string]string, other map[string]string, additionalOffset ...int) {
@@ -549,13 +616,16 @@ func testMapContainsOtherWithGomega(g Gomega, mapName string, base map[string]st
 	}
 }
 
-func testACLEnvVars(actualEnvVars []corev1.EnvVar, hasReadOnly bool, additionalOffset ...int) {
-	testACLEnvVarsWithGomega(Default, actualEnvVars, hasReadOnly, resolveOffset(additionalOffset))
+func insertExpectedAclEnvVars(dest map[string]string, hasReadOnly bool) {
+	expectedEnvVars := getExpectedAclEnvVars(hasReadOnly)
+	for _, expectedEnvVar := range expectedEnvVars {
+		dest[expectedEnvVar.Name] = expectedEnvVar.Value
+	}
 }
 
-func testACLEnvVarsWithGomega(g Gomega, actualEnvVars []corev1.EnvVar, hasReadOnly bool, additionalOffset ...int) {
+func getExpectedAclEnvVars(hasReadOnly bool) []corev1.EnvVar {
 	/*
-		This test verifies ACL related env vars are set correctly and in the correct order, but expects a very specific config to be used in your test SolrCloud config:
+		Populates ACL related env vars are set correctly and in the correct order, assuming a very specific test SolrCloud config:
 		set hasReadOnly = false if ReadOnlyACL is not provided
 					AllACL: &solrv1beta1.ZookeeperACL{
 						SecretRef:   "secret-name",
@@ -627,6 +697,11 @@ func testACLEnvVarsWithGomega(g Gomega, actualEnvVars []corev1.EnvVar, hasReadOn
 				ValueFrom: nil,
 			})
 	}
+	return zkAclEnvVars
+}
+
+func testACLEnvVarsWithGomega(g Gomega, actualEnvVars []corev1.EnvVar, hasReadOnly bool, additionalOffset ...int) {
+	zkAclEnvVars := getExpectedAclEnvVars(hasReadOnly)
 	g.ExpectWithOffset(resolveOffset(additionalOffset), actualEnvVars).To(Equal(zkAclEnvVars), "ZK ACL Env Vars are not correct")
 }
 
@@ -634,13 +709,13 @@ func cleanupTest(ctx context.Context, parentResource client.Object) {
 	cleanupObjects := []client.Object{
 		// Solr Operator CRDs, modify this list whenever CRDs are added/deleted
 		&solrv1beta1.SolrCloud{}, &solrv1beta1.SolrBackup{}, &solrv1beta1.SolrPrometheusExporter{},
-		&zk_api.ZookeeperCluster{},
+		&zkApi.ZookeeperCluster{},
 
 		// All dependent Kubernetes types, in order of dependence (deployment then replicaSet then pod)
 		&corev1.ConfigMap{}, &netv1.Ingress{},
 		&corev1.PersistentVolumeClaim{}, &corev1.PersistentVolume{},
 		&appsv1.StatefulSet{}, &appsv1.Deployment{}, &appsv1.ReplicaSet{}, &corev1.Pod{}, &corev1.PersistentVolumeClaim{},
-		&corev1.Secret{},
+		&corev1.Secret{}, &policyv1.PodDisruptionBudget{},
 	}
 	By("deleting all managed resources")
 	for _, obj := range cleanupObjects {
@@ -652,7 +727,7 @@ func cleanupTest(ctx context.Context, parentResource client.Object) {
 	serviceList := &corev1.ServiceList{}
 	Expect(k8sClient.List(ctx, serviceList, client.InNamespace(parentResource.GetNamespace()))).To(Succeed(), "List all of the services to delete in the namespace")
 	for _, item := range serviceList.Items {
-		Expect(k8sClient.Delete(ctx, &item)).To(Succeed())
+		Expect(k8sClient.Delete(ctx, &item)).To(Or(Succeed(), MatchError("services \""+item.Name+"\" not found")))
 	}
 }
 
@@ -736,9 +811,9 @@ var (
 		"testS4": "valueS4",
 	}
 	testNodeSelectors = map[string]string{
-		"beta.kubernetes.io/arch": "amd64",
-		"beta.kubernetes.io/os":   "linux",
-		"solrclouds":              "true",
+		"kubernetes.io/arch": "amd64",
+		"kubernetes.io/os":   "linux",
+		"solrclouds":         "true",
 	}
 	testProbeLivenessNonDefaults = &corev1.Probe{
 		InitialDelaySeconds: 20,
@@ -746,7 +821,7 @@ var (
 		SuccessThreshold:    1,
 		FailureThreshold:    3,
 		PeriodSeconds:       10,
-		Handler: corev1.Handler{
+		ProbeHandler: corev1.ProbeHandler{
 			HTTPGet: &corev1.HTTPGetAction{
 				Scheme: corev1.URISchemeHTTP,
 				Path:   "/solr/admin/info/system",
@@ -760,7 +835,7 @@ var (
 		SuccessThreshold:    1,
 		FailureThreshold:    3,
 		PeriodSeconds:       5,
-		Handler: corev1.Handler{
+		ProbeHandler: corev1.ProbeHandler{
 			TCPSocket: &corev1.TCPSocketAction{
 				Port: intstr.FromInt(8983),
 			},
@@ -772,7 +847,7 @@ var (
 		SuccessThreshold:    1,
 		FailureThreshold:    5,
 		PeriodSeconds:       5,
-		Handler: corev1.Handler{
+		ProbeHandler: corev1.ProbeHandler{
 			Exec: &corev1.ExecAction{
 				Command: []string{
 					"ls",
@@ -781,12 +856,12 @@ var (
 		},
 	}
 	testLifecycle = &corev1.Lifecycle{
-		PostStart: &corev1.Handler{
+		PostStart: &corev1.LifecycleHandler{
 			Exec: &corev1.ExecAction{
 				Command: []string{"/bin/sh", "-c", "echo Hello from the postStart handler"},
 			},
 		},
-		PreStop: &corev1.Handler{
+		PreStop: &corev1.LifecycleHandler{
 			Exec: &corev1.ExecAction{
 				Command: []string{"/bin/sh", "-c", "echo Hello from the preStop handler"},
 			},
@@ -808,6 +883,7 @@ var (
 	}
 	testPriorityClass              = "p4"
 	testImagePullSecretName        = "MAIN_SECRET"
+	testImagePullSecretName2       = "ANOTHER_SECRET"
 	testAdditionalImagePullSecrets = []corev1.LocalObjectReference{
 		{Name: "ADDITIONAL_SECRET_1"},
 		{Name: "ADDITIONAL_SECRET_2"},
@@ -830,11 +906,13 @@ var (
 	}
 	one                    = int64(1)
 	two                    = int64(2)
+	three                  = int64(3)
 	four                   = int32(4)
 	five                   = int32(5)
 	testPodSecurityContext = corev1.PodSecurityContext{
 		RunAsUser:  &one,
 		RunAsGroup: &two,
+		FSGroup:    &three,
 	}
 	extraVolumes = []solrv1beta1.AdditionalVolume{
 		{
@@ -862,12 +940,20 @@ var (
 	}
 	testResources = corev1.ResourceRequirements{
 		Limits: map[corev1.ResourceName]resource.Quantity{
-			corev1.ResourceCPU: *resource.NewMilliQuantity(5300, resource.DecimalSI),
+			corev1.ResourceCPU: resource.MustParse("5300m"),
 		},
 		Requests: map[corev1.ResourceName]resource.Quantity{
 			corev1.ResourceEphemeralStorage: resource.MustParse("5Gi"),
 		},
 	}
+	testResources2 = corev1.ResourceRequirements{
+		Limits: map[corev1.ResourceName]resource.Quantity{
+			corev1.ResourceCPU: resource.MustParse("400m"),
+		},
+		Requests: map[corev1.ResourceName]resource.Quantity{
+			corev1.ResourceEphemeralStorage: resource.MustParse("3Gi"),
+		},
+	}
 	extraContainers1 = []corev1.Container{
 		{
 			Name:                     "container1",
@@ -926,4 +1012,55 @@ var (
 		},
 	}
 	testIngressClass = "test-ingress-class"
+	testSolrZKOpts   = "-Dsolr.zk.opts=this"
+	testSolrOpts     = "-Dsolr.opts=this"
+	testZkProbes     = zkApi.Probes{
+		ReadinessProbe: &zkApi.Probe{
+			PeriodSeconds:    3,
+			SuccessThreshold: 5,
+			TimeoutSeconds:   10,
+		},
+		LivenessProbe: &zkApi.Probe{
+			InitialDelaySeconds: 6,
+			PeriodSeconds:       4,
+			FailureThreshold:    0,
+			SuccessThreshold:    3,
+			TimeoutSeconds:      0,
+		},
+	}
+	testZKAdminServerServicePolicy = zkApi.AdminServerServicePolicy{
+		External: true,
+		Annotations: map[string]string{
+			"testZKAS1": "valueS1",
+			"testZKAS2": "valueS2",
+		},
+	}
+	testZKClientServicePolicy = zkApi.ClientServicePolicy{
+		Annotations: map[string]string{
+			"testZKCS1": "valueS1",
+			"testZKCS2": "valueS2",
+		},
+	}
+	testZKHeadlessServicePolicy = zkApi.HeadlessServicePolicy{
+		Annotations: map[string]string{
+			"testZKCS1": "valueS1",
+			"testZKCS2": "valueS2",
+		},
+	}
+	testZkVolumes = []corev1.Volume{
+		{
+			Name: "vol1",
+			VolumeSource: corev1.VolumeSource{
+				EmptyDir: &corev1.EmptyDirVolumeSource{},
+			},
+		},
+	}
+	testZkVolumeMounts = []corev1.VolumeMount{
+		{
+			Name:      "vol1",
+			ReadOnly:  false,
+			MountPath: "/test/mount/path",
+			SubPath:   "sub/",
+		},
+	}
 )
